import { DefaultPlatform, PreparedEntity, SQLConnection, SQLConnectionPool, SQLDatabaseAdapter, SQLDatabaseQuery, SQLDatabaseQueryFactory, SQLPersistence, SQLQueryModel, SQLQueryResolver, SQLStatement } from '@deepkit/sql';
import { DatabasePersistenceChangeSet, DatabaseSession, DatabaseTransaction, DeleteResult, OrmEntity, PatchResult } from '@deepkit/orm';
import { PostgresPlatform } from './postgres-platform.js';
import type { Pool, PoolClient, PoolConfig } from 'pg';
import pg from 'pg';
import { AbstractClassType, ClassType } from '@deepkit/core';
import { Stopwatch } from '@deepkit/stopwatch';
import { Changes, ReceiveType, ReflectionClass } from '@deepkit/type';
import { Logger } from '@deepkit/logger';
export declare class PostgresStatement extends SQLStatement {
    protected logger: Logger;
    protected sql: string;
    protected client: PoolClient;
    protected stopwatch?: Stopwatch | undefined;
    protected released: boolean;
    constructor(logger: Logger, sql: string, client: PoolClient, stopwatch?: Stopwatch | undefined);
    get(params?: any[]): Promise<any>;
    all(params?: any[]): Promise<any>;
    release(): void;
}
export declare class PostgresConnection extends SQLConnection {
    connection: PoolClient;
    protected changes: number;
    lastReturningRows: any[];
    constructor(connectionPool: PostgresConnectionPool, connection: PoolClient, logger: Logger, transaction?: DatabaseTransaction, stopwatch?: Stopwatch);
    prepare(sql: string): Promise<PostgresStatement>;
    run(sql: string, params?: any[]): Promise<void>;
    getChanges(): Promise<number>;
}
export type TransactionTypes = 'REPEATABLE READ' | 'READ COMMITTED' | 'SERIALIZABLE';
export declare class PostgresDatabaseTransaction extends DatabaseTransaction {
    connection?: PostgresConnection;
    setTransaction?: TransactionTypes;
    /**
     * This is the default for mysql databases.
     */
    repeatableRead(): this;
    readCommitted(): this;
    serializable(): this;
    begin(): Promise<void>;
    commit(): Promise<void>;
    rollback(): Promise<void>;
}
export declare class PostgresConnectionPool extends SQLConnectionPool {
    protected pool: Pool;
    protected logger: Logger;
    protected stopwatch?: Stopwatch | undefined;
    constructor(pool: Pool, logger: Logger, stopwatch?: Stopwatch | undefined);
    getConnection(transaction?: PostgresDatabaseTransaction): Promise<PostgresConnection>;
    release(connection: PostgresConnection): void;
}
export declare class PostgresPersistence extends SQLPersistence {
    protected platform: DefaultPlatform;
    connectionPool: PostgresConnectionPool;
    constructor(platform: DefaultPlatform, connectionPool: PostgresConnectionPool, session: DatabaseSession<any>);
    handleSpecificError(error: Error): Error;
    batchUpdate<T extends OrmEntity>(entity: PreparedEntity, changeSets: DatabasePersistenceChangeSet<T>[]): Promise<void>;
    protected populateAutoIncrementFields<T>(classSchema: ReflectionClass<T>, items: T[]): Promise<void>;
    protected getInsertSQL(classSchema: ReflectionClass<any>, fields: string[], values: string[]): string;
}
export declare class PostgresSQLQueryResolver<T extends OrmEntity> extends SQLQueryResolver<T> {
    delete(model: SQLQueryModel<T>, deleteResult: DeleteResult<T>): Promise<void>;
    handleSpecificError(error: Error): Error;
    patch(model: SQLQueryModel<T>, changes: Changes<T>, patchResult: PatchResult<T>): Promise<void>;
}
export declare class PostgresSQLDatabaseQuery<T extends OrmEntity> extends SQLDatabaseQuery<T> {
}
export declare class PostgresSQLDatabaseQueryFactory extends SQLDatabaseQueryFactory {
    createQuery<T extends OrmEntity>(type?: ReceiveType<T> | ClassType<T> | AbstractClassType<T> | ReflectionClass<T>): PostgresSQLDatabaseQuery<T>;
}
export declare class PostgresDatabaseAdapter extends SQLDatabaseAdapter {
    protected options: PoolConfig;
    protected pool: pg.Pool;
    connectionPool: PostgresConnectionPool;
    platform: PostgresPlatform;
    closed: boolean;
    constructor(options: PoolConfig | string, additional?: Partial<PoolConfig>);
    setLogger(logger: Logger): void;
    getName(): string;
    getSchemaName(): string;
    createPersistence(session: DatabaseSession<this>): SQLPersistence;
    createTransaction(session: DatabaseSession<this>): PostgresDatabaseTransaction;
    queryFactory(session: DatabaseSession<any>): SQLDatabaseQueryFactory;
    disconnect(force?: boolean): void;
}
